@@ -8,7 +8,10 @@ from elegy import types, utils
 from elegy.losses.loss import Loss, Reduction
 
 
-def smooth_labels(labels: jnp.ndarray, smoothing: jnp.ndarray,) -> jnp.ndarray:
+def smooth_labels(
+    labels: jnp.ndarray,
+    smoothing: jnp.ndarray,
+) -> jnp.ndarray:
     smooth_positives = 1.0 - smoothing
     smooth_negatives = smoothing / labels.shape[-1]
     return smooth_positives * labels + smooth_negatives
@@ -28,7 +31,7 @@ def categorical_crossentropy(
         y_pred = jax.nn.log_softmax(y_pred)
 
     else:
-        y_pred = jnp.maximum(y_pred, utils.EPSILON)
+        y_pred = jnp.maximum(y_pred, types.EPSILON)
         y_pred = jnp.log(y_pred)
 
     return -jnp.sum(y_true * y_pred, axis=-1)
@@ -44,7 +47,7 @@ class CategoricalCrossentropy(Loss):
     In the snippet below, there is `# classes` floating pointing values per
     example. The shape of both `y_pred` and `y_true` are
     `[batch_size, num_classes]`.
-    
+
     Usage:
     ```python
     y_true = jnp.array([[0, 1, 0], [0, 0, 1]])
@@ -74,8 +77,8 @@ class CategoricalCrossentropy(Loss):
     model = elegy.Model(
         module_fn,
         loss=elegy.losses.CategoricalCrossentropy(),
-        metrics=elegy.metrics.Accuracy.defer(),
-        optimizer=optix.adam(1e-3),
+        metrics=elegy.metrics.Accuracy(),
+        optimizer=optax.adam(1e-3),
     )
     ```
     """
@@ -85,13 +88,13 @@ class CategoricalCrossentropy(Loss):
         from_logits: bool = False,
         label_smoothing: float = 0,
         reduction: tp.Optional[Reduction] = None,
-        name: tp.Optional[str] = None,
         weight: tp.Optional[float] = None,
         on: tp.Optional[types.IndexLike] = None,
+        **kwargs,
     ):
         """
         Initializes `CategoricalCrossentropy` instance.
-        
+
         Arguments:
             from_logits: Whether `y_pred` is expected to be a logits tensor. By
                 default, we assume that `y_pred` encodes a probability distribution.
@@ -104,7 +107,6 @@ class CategoricalCrossentropy(Loss):
                 loss. Default value is `SUM_OVER_BATCH_SIZE`. Indicates that the reduction
                 option will be determined by the usage context. For almost all cases
                 this defaults to `SUM_OVER_BATCH_SIZE`.
-            name: Optional name for the op.
             weight: Optional weight contribution for the total loss. Defaults to `1`.
             on: A string or integer, or iterable of string or integers, that
                 indicate how to index/filter the `y_true` and `y_pred`
@@ -114,7 +116,7 @@ class CategoricalCrossentropy(Loss):
                 then `y_true = y_true["a"][0]["b"]`, same for `y_pred`. For more information
                 check out [Keras-like behavior](https://poets-ai.github.io/elegy/guides/modules-losses-metrics/#keras-like-behavior).
         """
-        super().__init__(reduction=reduction, name=name, weight=weight, on=on)
+        super().__init__(reduction=reduction, weight=weight, on=on, **kwargs)
 
         self._from_logits = from_logits
         self._label_smoothing = label_smoothing
@@ -127,7 +129,7 @@ class CategoricalCrossentropy(Loss):
     ) -> jnp.ndarray:
         """
         Invokes the `CategoricalCrossentropy` instance.
-        
+
         Arguments:
             y_true: Ground truth values.
             y_pred: The predicted values.
