@@ -1,6 +1,6 @@
 import elegy
-import haiku as hk
-from haiku.testing import transform_and_run
+
+
 import jax.numpy as jnp
 import jax
 
@@ -12,7 +12,7 @@ import jax
 # debugpy.wait_for_client()
 
 
-@transform_and_run
+#
 def test_basic():
 
     y_true = jnp.array([[0.0, 1.0], [0.0, 0.0]])
@@ -37,6 +37,7 @@ def test_basic():
     assert list(mse(y_true, y_pred)) == [0.5, 0.5]
 
 
+#
 def test_function():
 
     rng = jax.random.PRNGKey(42)
@@ -51,7 +52,7 @@ def test_function():
     assert jnp.array_equal(loss, jnp.mean(jnp.square(y_true - y_pred), axis=-1))
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
 
     test_basic()
-    test_function()
\ No newline at end of file
+    test_function()
