@@ -4,27 +4,38 @@ import typing as tp
 import jax
 import jax.numpy as jnp
 
-from elegy import utils
+from elegy import utils, types
 from elegy.losses.loss import Loss, Reduction
 from elegy.losses.categorical_crossentropy import categorical_crossentropy
 
 
 def sparse_categorical_crossentropy(
-    y_true: jnp.ndarray, y_pred: jnp.ndarray, from_logits: bool = False
+    y_true: jnp.ndarray,
+    y_pred: jnp.ndarray,
+    from_logits: bool = False,
+    check_bounds: bool = True,
 ) -> jnp.ndarray:
 
+    n_classes = y_pred.shape[-1]
+
     if from_logits:
         y_pred = jax.nn.log_softmax(y_pred)
-        return -jnp.take_along_axis(y_pred, y_true[..., None], axis=-1)[..., 0]
-
+        loss = -jnp.take_along_axis(y_pred, y_true[..., None], axis=-1)[..., 0]
     else:
         # select output value
         y_pred = jnp.take_along_axis(y_pred, y_true[..., None], axis=-1)[..., 0]
 
         # calculate log
-        y_pred = jnp.maximum(y_pred, utils.EPSILON)
+        y_pred = jnp.maximum(y_pred, types.EPSILON)
         y_pred = jnp.log(y_pred)
-        return -y_pred
+        loss = -y_pred
+
+    if check_bounds:
+        # set NaN where y_true is negative or larger/equal to the number of y_pred channels
+        loss = jnp.where(y_true < 0, jnp.nan, loss)
+        loss = jnp.where(y_true >= n_classes, jnp.nan, loss)
+
+    return loss
 
 
 class SparseCategoricalCrossentropy(Loss):
@@ -71,15 +82,15 @@ class SparseCategoricalCrossentropy(Loss):
     ```
 
     Usage with the `Elegy` API:
-    
+
     ```python
     model = elegy.Model(
         module_fn,
-        loss=lelegy.losses.SparseCategoricalCrossentropy(),
-        metrics=lelegy.metrics.Accuracy.defer(),
-        optimizer=optix.adam(1e-3),
+        loss=elegy.losses.SparseCategoricalCrossentropy(),
+        metrics=elegy.metrics.Accuracy(),
+        optimizer=optax.adam(1e-3),
     )
-    
+
     ```
     """
 
@@ -87,13 +98,14 @@ class SparseCategoricalCrossentropy(Loss):
         self,
         from_logits: bool = False,
         reduction: tp.Optional[Reduction] = None,
-        name: tp.Optional[str] = None,
         weight: tp.Optional[float] = None,
         on: tp.Optional[types.IndexLike] = None,
+        check_bounds: tp.Optional[bool] = True,
+        **kwargs
     ):
         """
         Initializes `SparseCategoricalCrossentropy` instance.
-        
+
         Arguments:
             from_logits: Whether `y_pred` is expected to be a logits tensor. By
                 default, we assume that `y_pred` encodes a probability distribution.
@@ -101,7 +113,6 @@ class SparseCategoricalCrossentropy(Loss):
             reduction: (Optional) Type of `elegy.losses.Reduction` to apply to
                 loss. Default value is `SUM_OVER_BATCH_SIZE`. For almost all cases
                 this defaults to `SUM_OVER_BATCH_SIZE`.
-            name: Optional name for the op. Defaults to 'sparse_categorical_crossentropy'.
             weight: Optional weight contribution for the total loss. Defaults to `1`.
             on: A string or integer, or iterable of string or integers, that
                 indicate how to index/filter the `y_true` and `y_pred`
@@ -110,17 +121,22 @@ class SparseCategoricalCrossentropy(Loss):
                 the structures will be indexed iteratively, for example if `on = ["a", 0, "b"]`
                 then `y_true = y_true["a"][0]["b"]`, same for `y_pred`. For more information
                 check out [Keras-like behavior](https://poets-ai.github.io/elegy/guides/modules-losses-metrics/#keras-like-behavior).
+            check_bounds: If `True` (default), checks `y_true` for negative values and values
+                larger or equal than the number of channels in `y_pred`. Sets loss to NaN
+                if this is the case. If `False`, the check is disabled and the loss may contain
+                incorrect values.
         """
-        super().__init__(reduction=reduction, name=name, weight=weight, on=on)
+        super().__init__(reduction=reduction, weight=weight, on=on, **kwargs)
 
         self._from_logits = from_logits
+        self._check_bounds = check_bounds
 
     def call(
         self, y_true, y_pred, sample_weight: tp.Optional[jnp.ndarray] = None
     ) -> jnp.ndarray:
         """
         Invokes the `SparseCategoricalCrossentropy` instance.
-        
+
         Arguments:
             y_true: Ground truth values.
             y_pred: The predicted values.
@@ -133,11 +149,14 @@ class SparseCategoricalCrossentropy(Loss):
                 broadcasted to this shape), then each loss element of `y_pred` is scaled
                 by the corresponding value of `sample_weight`. (Note on`dN-1`: all loss
                 functions reduce by 1 dimension, usually axis=-1.)
-        
+
         Returns:
             Loss values per sample.
         """
 
         return sparse_categorical_crossentropy(
-            y_true, y_pred, from_logits=self._from_logits
+            y_true,
+            y_pred,
+            from_logits=self._from_logits,
+            check_bounds=self._check_bounds,
         )
