@@ -4,13 +4,13 @@ import typing as tp
 
 import jax.numpy as jnp
 
-from elegy.metrics import reduce
+from elegy.metrics.reduce import Reduce, Reduction
 
 
-class Mean(reduce.Reduce):
+class Mean(Reduce):
     """
     Computes the (weighted) mean of the given values.
-    
+
     For example, if values is `[1, 3, 5, 7]` then the mean is `4`.
     If the weights were specified as `[1, 1, 0, 0]` then the mean would be `2`.
     This metric creates two variables, `total` and `count` that are used to
@@ -18,7 +18,7 @@ class Mean(reduce.Reduce):
     which is an idempotent operation that simply divides `total` by `count`.
     If `sample_weight` is `None`, weights default to 1.
     Use `sample_weight` of 0 to mask values.
-  
+
     Usage:
 
     ```python
@@ -32,48 +32,40 @@ class Mean(reduce.Reduce):
     ```
 
     Usage with elegy API:
-    
+
     ```python
     model = elegy.Model(
         module_fn,
         loss=elegy.losses.MeanSquaredError(),
-        metrics=elegy.metrics.Mean.defer(),
+        metrics=elegy.metrics.Mean(),
     )
     ```
     """
 
-    def __init__(
-        self,
-        name: tp.Optional[str] = None,
-        dtype: tp.Optional[jnp.dtype] = None,
-        on: tp.Optional[types.IndexLike] = None,
-    ):
+    def __init__(self, on: tp.Optional[types.IndexLike] = None, **kwargs):
         """Creates a `Mean` instance.
         Arguments:
-          name: (Optional) string name of the metric instance.
-          dtype: (Optional) data type of the metric result.
-          on: A string or integer, or iterable of string or integers, that
+            on: A string or integer, or iterable of string or integers, that
                 indicate how to index/filter the `y_true` and `y_pred`
                 arguments before passing them to `call`. For example if `on = "a"` then
                 `y_true = y_true["a"]`. If `on` is an iterable
                 the structures will be indexed iteratively, for example if `on = ["a", 0, "b"]`
                 then `y_true = y_true["a"][0]["b"]`, same for `y_pred`. For more information
                 check out [Keras-like behavior](https://poets-ai.github.io/elegy/guides/modules-losses-metrics/#keras-like-behavior).
+            kwargs: Additional keyword arguments passed to Module.
         """
-        super().__init__(
-            reduction=reduce.Reduction.WEIGHTED_MEAN, name=name, dtype=dtype, on=on
-        )
+        super().__init__(reduction=Reduction.WEIGHTED_MEAN, on=on, **kwargs)
 
     def call(
         self, values: jnp.ndarray, sample_weight: tp.Optional[jnp.ndarray] = None
     ) -> jnp.ndarray:
         """
         Accumulates the mean statistic over various batches.
-        
+
         Arguments:
             values: Per-example value.
             sample_weight: Optional weighting of each example.
-        
+
         Returns:
             Array with the cumulative mean.
         """
