@@ -162,6 +162,12 @@ class Model(ModelBase):
 
         super().__init__(**kwargs)
 
+        # maybe add rng if initialized
+        if self.initialized and (
+            not hasattr(self.states, "rng") or self.states.rng is None
+        ):
+            self.states = self.states.update(rng=types.RNGSeq(seed))
+
         self.module = module
         self.loss = loss
         self.metrics = metrics
@@ -201,8 +207,9 @@ class Model(ModelBase):
     # ----------------------------------------------------------------
     # implement low-level API methods
     # ----------------------------------------------------------------
-    def base_states(self) -> types.States:
-        return types.States(
+    def states_step(self) -> types.States:
+        states = super().states_step()
+        return states.update(
             rng=types.RNGSeq(self.seed),
             net_params=None,
             net_states=None,
@@ -210,15 +217,45 @@ class Model(ModelBase):
             optimizer_states=None,
         )
 
+    def init_step(
+        self,
+        x: tp.Any,
+        y_true: tp.Any,
+        sample_weight: tp.Optional[np.ndarray],
+        class_weight: tp.Optional[np.ndarray],
+        states: types.States,
+    ) -> types.States:
+
+        states = states.maybe_update(**self.states_step())
+        training = True
+        initializing = True
+
+        _, states = utils.inject_dependencies(self.train_step)(
+            x=x,
+            y_true=y_true,
+            sample_weight=sample_weight,
+            class_weight=class_weight,
+            states=states,
+            initializing=initializing,
+            training=training,
+        )
+
+        return states
+
     def summary_step(
         self,
         x: tp.Any,
         states: types.States,
-        initializing: bool,
-        training: bool,
     ) -> tp.List[types.SummaryTableEntry]:
+        initializing = True
+        training = True
+
+        states = states.maybe_update(**self.states_step())
+
         with hooks.context(summaries=True):
-            self.call_pred_step(x, states, initializing, training)
+            _, states = utils.inject_dependencies(self.pred_step)(
+                x, states, initializing, training
+            )
             summaries = hooks.get_summaries()
 
         entries: tp.List[types.SummaryTableEntry] = []
@@ -229,8 +266,8 @@ class Model(ModelBase):
                 path=path,
                 module=module,
                 value=value,
-                net_params=self.states.net_params,
-                net_states=self.states.net_states,
+                net_params=states.net_params,
+                net_states=states.net_states,
             )
 
             entries.append(
@@ -326,7 +363,7 @@ class Model(ModelBase):
     ) -> model_core.TestStep:
 
         with hooks.context(losses=True, summaries=True):
-            y_pred, states = self.call_pred_step(
+            y_pred, states = utils.inject_dependencies(self.pred_step)(
                 x=x,
                 states=states,
                 initializing=initializing,
@@ -398,7 +435,7 @@ class Model(ModelBase):
             class_weight: tp.Optional[np.ndarray],
         ):
             states = states.update(net_params=net_params)
-            loss, logs, states = self.call_test_step(
+            loss, logs, states = utils.inject_dependencies(self.test_step)(
                 x=x,
                 y_true=y_true,
                 states=states,
@@ -433,7 +470,7 @@ class Model(ModelBase):
     ) -> model_core.TrainStep:
 
         if initializing:
-            loss, logs, states = self.call_test_step(
+            loss, logs, states = utils.inject_dependencies(self.test_step)(
                 x=x,
                 y_true=y_true,
                 states=states,
@@ -445,7 +482,7 @@ class Model(ModelBase):
             grads = None
 
         else:
-            loss, logs, states, grads = self.call_grad_step(
+            loss, logs, states, grads = utils.inject_dependencies(self.grad_step)(
                 x=x,
                 y_true=y_true,
                 states=states,
@@ -455,7 +492,9 @@ class Model(ModelBase):
                 training=training,
             )
 
-        if self.optimizer is None:
+        if initializing and self.optimizer is None:
+            return model_core.TrainStep(logs, states)
+        elif self.optimizer is None:
             raise types.MissingOptimizer(
                 "Trying to run `train_step` without an optimizer, "
                 "please provide an optimizer to the Model(...) constructor or "
