@@ -1,14 +1,21 @@
 # Implementation based on tf.keras.engine.training.py
 # https://github.com/tensorflow/tensorflow/blob/v2.2.0/tensorflow/python/keras/engine/training.py
 
+import io
 import pickle
 import typing as tp
 from copy import copy
+from io import StringIO
 from pathlib import Path
 
+import jax
 import jax.numpy as jnp
 import numpy as np
-from elegy import types
+import rich
+import rich.box
+import toolz
+import yaml
+from elegy import types, utils
 from elegy.callbacks import Callback, CallbackList, History
 from elegy.data import (
     DataHandler,
@@ -17,8 +24,10 @@ from elegy.data import (
     train_validation_split,
     unpack_x_y_sample_weight,
 )
+from elegy.data import utils as data_utils
 from elegy.model.model_core import ModelCore, PredStep, TestStep
-from tabulate import tabulate
+from rich.table import Table
+from rich.text import Text
 
 # from elegy.module import Module
 
@@ -117,30 +126,196 @@ class ModelBase(ModelCore):
         "states",
     ]
 
-    def pred_step(self, *args, **kwargs):
-        raise types.MissingMethod()
+    def init(
+        self,
+        x: tp.Optional[tp.Any] = None,
+        y: tp.Optional[tp.Any] = None,
+        batch_size: tp.Optional[int] = None,
+        class_weight: tp.Optional[tp.Mapping[str, float]] = None,
+        sample_weight: tp.Optional[np.ndarray] = None,
+    ) -> None:
 
-    def test_step(self, *args, **kwargs):
-        raise types.MissingMethod()
+        if self.initialized:
+            return
 
-    def train_step(self, *args, **kwargs):
-        raise types.MissingMethod()
+        if x is None:
+            x = {}
+
+        data_handler = DataHandler(
+            x=x,
+            y=y,
+            sample_weight=sample_weight,
+            batch_size=batch_size,
+            class_weight=class_weight,
+            steps_per_epoch=1,
+        )
+
+        epoch, iterator = next(data_handler.enumerate_epochs())
+        batch = next(iterator)
+        x_batch, y_batch, sample_weight = unpack_x_y_sample_weight(batch)
+
+        self.init_on_batch(
+            x=x_batch,
+            y_true=y_batch,
+            sample_weight=sample_weight,
+            class_weight=class_weight,
+        )
+
+    def summary(
+        self,
+        x: tp.Optional[tp.Any] = None,
+        depth: int = 2,
+        tablefmt: str = "fancy_grid",
+        return_repr: bool = False,
+        initialize: bool = False,
+        eval_shape: bool = True,
+        **tablulate_kwargs,
+    ) -> tp.Optional[str]:
+        """
+        Prints a summary of the network.
+        Arguments:
+            x: A sample of inputs to the network.
+            depth: The level number of nested level which will be showed.
+                Information about summaries from modules deeper than `depth`
+                will be aggregated together.
+            tablefmt: A string representing the style of the table generated by
+                `tabulate`. See
+                [python-tabulate](https://github.com/astanin/python-tabulate)
+                for more options.
+            tablulate_kwargs: Additional keyword arguments passed to `tabulate`.
+                See [python-tabulate](https://github.com/astanin/python-tabulate)
+                for more options.
+        """
+
+        if x is None:
+            x = {}
+
+        entries: tp.List[types.SummaryTableEntry]
+        states = self.states.copy() if self.run_eagerly else self.states
+
+        method = (
+            self.call_summary_step if self.run_eagerly else self.call_summary_step_jit
+        )
+
+        if eval_shape:
+            entries = jax.eval_shape(self.call_summary_step, x, states)
+        else:
+            entries = method(x, states)
+
+        total_entry = entries[-1]
+        entries = entries[:-1]
+
+        depth_groups: tp.Dict[str, tp.List[types.SummaryTableEntry]] = toolz.groupby(
+            lambda entry: "/".join(entry.path.split("/")[:depth]), entries
+        )
+
+        entries = [
+            utils.get_grouped_entry(entry, depth_groups)
+            for entry in entries
+            if entry.path in depth_groups
+        ]
+
+        main_table = Table(
+            show_header=True,
+            show_lines=True,
+            show_footer=True,
+            # box=rich.box.HORIZONTALS,
+        )
+
+        main_table.add_column("Layer")
+        main_table.add_column("Outputs Shape")
+        main_table.add_column("Trainable\nParameters")
+        main_table.add_column("Non-trainable\nParameters")
+
+        rows: tp.List[tp.List[str]] = []
+
+        rows.append(["Inputs", utils.format_output(x), "", ""])
+
+        for entry in entries:
+            rows.append(
+                [
+                    f"{entry.path}{{pad}}  "
+                    + (
+                        f"[dim]{entry.module_type_name}[/]"
+                        if entry.module_type_name
+                        else ""
+                    ),
+                    utils.format_output(entry.output_value),
+                    f"[green]{entry.trainable_params_count:,}[/]{{pad}}    {utils.format_size(entry.trainable_params_size)}"
+                    if entry.trainable_params_count > 0
+                    else "",
+                    f"[green]{entry.non_trainable_params_count:,}[/]{{pad}}    {utils.format_size(entry.non_trainable_params_size)}"
+                    if entry.non_trainable_params_count > 0
+                    else "",
+                ]
+            )
+
+        # global summaries
+        params_count = total_entry.trainable_params_count
+        params_size = total_entry.trainable_params_size
+        states_count = total_entry.non_trainable_params_count
+        states_size = total_entry.non_trainable_params_size
+        total_count = params_count + states_count
+        total_size = params_size + states_size
+
+        rows.append(
+            [
+                "",
+                "Total",
+                (
+                    f"[green]{params_count:,}[/]{{pad}}    {utils.format_size(params_size)}"
+                    if params_count > 0
+                    else ""
+                ),
+                (
+                    f"[green]{states_count:,}[/]{{pad}}    {utils.format_size(states_size)}"
+                    if states_count > 0
+                    else ""
+                ),
+            ]
+        )
+
+        # add padding
+        for col in range(4):
+            max_length = max(
+                len(line.split("{pad}")[0])
+                for row in rows
+                for line in row[col].split("\n")
+            )
+
+            for row in rows:
+                row[col] = "\n".join(
+                    line.format(
+                        pad=" " * (max_length - len(line.rstrip().split("{pad}")[0]))
+                    )
+                    for line in row[col].rstrip().split("\n")
+                )
+
+        for row in rows[:-1]:
+            main_table.add_row(*row)
+
+        main_table.columns[1].footer = Text.from_markup(rows[-1][1], justify="right")
+        main_table.columns[2].footer = rows[-1][2]
+        main_table.columns[3].footer = rows[-1][3]
+        main_table.caption_style = "bold"
+        main_table.caption = (
+            "\nTotal Parameters: "
+            + f"[green]{total_count:,}[/]   {utils.format_size(total_size)}"
+            if total_count > 0
+            else ""
+        )
+
+        summary = "\n" + utils.get_table_repr(main_table)
+
+        print(summary)
+
+        if return_repr:
+            return summary
 
     def fit(
         self,
-        x: tp.Union[
-            np.ndarray,
-            tp.Mapping[str, np.ndarray],
-            tp.Tuple[np.ndarray],
-            tp.Iterable,
-            None,
-        ] = None,
-        y: tp.Union[
-            np.ndarray,
-            tp.Mapping[str, np.ndarray],
-            tp.Tuple[np.ndarray],
-            None,
-        ] = None,
+        x: tp.Optional[tp.Any] = None,
+        y: tp.Optional[tp.Any] = None,
         batch_size: tp.Optional[int] = None,
         epochs: int = 1,
         verbose: int = 1,
@@ -155,6 +330,7 @@ class ModelBase(ModelCore):
         validation_steps: tp.Optional[int] = None,
         validation_batch_size: tp.Optional[int] = None,
         validation_freq: int = 1,
+        drop_remaining: bool = True,
     ) -> History:
         """
         Trains the model for a fixed number of epochs (iterations on a dataset).
@@ -309,6 +485,15 @@ class ModelBase(ModelCore):
         if x is None:
             x = {}
 
+        if not self.initialized:
+            self.init(
+                x=x,
+                y=y,
+                batch_size=batch_size,
+                class_weight=class_weight,
+                sample_weight=sample_weight,
+            )
+
         if validation_split:
             # Create the validation data using the training data. Only supported for
             # `Jax Numpy` and `NumPy` input.
@@ -343,6 +528,7 @@ class ModelBase(ModelCore):
         callbacks.on_train_begin()
         # data_handler._initial_epoch = (  # pylint: disable=protected-access
         #     self._maybe_load_initial_epoch_from_ckpt(initial_epoch))
+        epoch_logs = {}
 
         for epoch, iterator in data_handler.enumerate_epochs():
             self.reset_metrics()
@@ -355,6 +541,11 @@ class ModelBase(ModelCore):
                     # sample_weight = batch[2] if len(batch) == 3 else None
                     x_batch, y_batch, sample_weight = unpack_x_y_sample_weight(batch)
 
+                    if drop_remaining and not data_utils.has_batch_size(
+                        batch, data_handler.batch_size
+                    ):
+                        continue
+
                     tmp_logs = self.train_on_batch(
                         x=x_batch,
                         y=y_batch,
@@ -367,11 +558,18 @@ class ModelBase(ModelCore):
                     logs = tmp_logs
                     callbacks.on_train_batch_end(step, logs)
 
+                    if self.stop_training:
+                        break
+
             epoch_logs = copy(logs)
             epoch_logs.update({"size": data_handler.batch_size})
 
             # Run validation.
-            if validation_data and self._should_eval(epoch, validation_freq):
+            if (
+                validation_data
+                and self._should_eval(epoch, validation_freq)
+                and not self.stop_training
+            ):
                 val_x, val_y, val_sample_weight = unpack_x_y_sample_weight(
                     validation_data
                 )
@@ -384,6 +582,7 @@ class ModelBase(ModelCore):
                         steps=validation_steps,
                         callbacks=callbacks,
                         # return_dict=True,
+                        drop_remaining=drop_remaining,
                     )
 
                     val_logs = {"val_" + name: val for name, val in val_logs.items()}
@@ -392,37 +591,25 @@ class ModelBase(ModelCore):
                     pass
 
             callbacks.on_epoch_end(epoch, epoch_logs)
-            # print(
-            #     f"epoch: {epoch} - "
-            #     + " - ".join(f"{key}: {value:.3f}" for key, value in epoch_logs.items())
-            # )
+
             if self.stop_training:
                 break
 
-        callbacks.on_train_end()
+        callbacks.on_train_end(epoch_logs)
 
         return self.history
 
     def evaluate(
         self,
-        x: tp.Union[
-            np.ndarray,
-            tp.Mapping[str, np.ndarray],
-            tp.Tuple[np.ndarray],
-            tp.Iterable,
-        ],
-        y: tp.Union[
-            jnp.ndarray,
-            np.ndarray,
-            tp.Mapping[str, np.ndarray],
-            tp.Tuple[np.ndarray],
-            None,
-        ] = None,
+        x: tp.Optional[tp.Any] = None,
+        y: tp.Optional[tp.Any] = None,
         verbose: int = 1,
         batch_size: tp.Optional[int] = None,
         sample_weight: tp.Optional[np.ndarray] = None,
+        class_weight: tp.Optional[tp.Mapping[str, float]] = None,
         steps: tp.Optional[int] = None,
         callbacks: tp.Union[tp.List[Callback], CallbackList, None] = None,
+        drop_remaining: bool = False,
     ) -> types.Logs:
         """Returns the loss value & metrics values for the model in test mode.
         Computation is done in batches.
@@ -475,6 +662,18 @@ class ModelBase(ModelCore):
             ValueError: in case of invalid arguments.
         """
 
+        if x is None:
+            x = {}
+
+        if not self.initialized:
+            self.init(
+                x=x,
+                y=y,
+                batch_size=batch_size,
+                class_weight=class_weight,
+                sample_weight=sample_weight,
+            )
+
         data_handler = DataHandler(
             x=x,
             y=y,
@@ -510,10 +709,16 @@ class ModelBase(ModelCore):
                     batch = next(iterator)
                     x_batch, y_batch, sample_weight = unpack_x_y_sample_weight(batch)
 
+                    if drop_remaining and not data_utils.has_batch_size(
+                        batch, data_handler.batch_size
+                    ):
+                        continue
+
                     tmp_logs = self.test_on_batch(
                         x=x_batch,
                         y=y_batch,
                         sample_weight=sample_weight,
+                        class_weight=None,
                     )
                     tmp_logs.update({"size": data_handler.batch_size})
                     logs = tmp_logs
@@ -525,16 +730,13 @@ class ModelBase(ModelCore):
 
     def predict(
         self,
-        x: tp.Union[
-            np.ndarray,
-            tp.Mapping[str, np.ndarray],
-            tp.Tuple[np.ndarray],
-            tp.Iterable,
-        ],
+        x: tp.Optional[tp.Any] = None,
         verbose: int = 0,
         batch_size: tp.Optional[int] = None,
         steps: tp.Optional[int] = None,
         callbacks: tp.Union[tp.List[Callback], CallbackList, None] = None,
+        drop_remaining: bool = False,
+        initialize: bool = False,
     ) -> tp.Any:
         """Generates output predictions for the input samples.
         Computation is done in batches.
@@ -579,6 +781,19 @@ class ModelBase(ModelCore):
                 that is not a multiple of the batch size.
         """
 
+        if x is None:
+            x = {}
+
+        if not self.initialized:
+            if initialize:
+                self.init(x=x, batch_size=batch_size)
+            else:
+                raise types.ModelNotInitialized(
+                    f"Model not initialized, please execute `model.init` or `model.init_on_batch` "
+                    "before running this method, or pass `initialize=True` to initialize with the available data "
+                    "(this might not initialize the optimizer)."
+                )
+
         outputs = None
 
         data_handler = DataHandler(
@@ -610,6 +825,12 @@ class ModelBase(ModelCore):
                 for step in data_handler.steps():
                     callbacks.on_predict_batch_begin(step)
                     batch = next(iterator)
+
+                    if drop_remaining and not data_utils.has_batch_size(
+                        batch, data_handler.batch_size
+                    ):
+                        continue
+
                     tmp_batch_outputs = self.predict_on_batch(x=batch[0])
                     batch_outputs = tmp_batch_outputs
 
