@@ -17,13 +17,7 @@ else:
     from typing_extensions import Protocol, runtime_checkable
 
 EPSILON = 1e-7
-
-
-class Mode(int, Enum):
-    none = 0
-    pred = 1
-    test = 2
-    train = 3
+F = tp.TypeVar("F", bound=tp.Callable)
 
 
 class TrivialPytree:
@@ -48,7 +42,9 @@ class RNGSeq(TrivialPytree):
 
     def __init__(self, key: tp.Union[int, jnp.ndarray]):
         self.key = (
-            jax.random.PRNGKey(key) if isinstance(key, int) or key.shape == () else key
+            jax.random.PRNGKey(key)
+            if isinstance(key, int) or (hasattr(key, "shape") and key.shape == ())
+            else key
         )
 
     def next(self) -> jnp.ndarray:
@@ -209,27 +205,44 @@ class States(tp.Mapping):
         return iter(self.__dict__)
 
     def __getattr__(self, key):
-        return object.__getattr__(self, key)
+        try:
+            return object.__getattr__(self, key)
+        except AttributeError:
+            raise AttributeError(
+                f"'{self.__class__.__name__}' has not attribute '{key}'"
+            )
 
     def __setattr__(self, key, value):
         raise AttributeError("can't set attribute")
 
     def update(self, **kwargs) -> "States":
+        """Returns a new States object, updating all attributes from kwargs."""
         data = self.__dict__.copy()
         data.update(kwargs)
         return States(data)
 
     def maybe_update(self, **kwargs) -> "States":
-
+        """Returns a new States object, updating attributes that are not yet present."""
         kwargs = {
             key: value
             for key, value in kwargs.items()
-            if key not in self.__dict__
-            or (self.__dict__[key] is None and value is not None)
+            if key not in self.__dict__ or self.__dict__[key] is None
         }
 
         return self.update(**kwargs)
 
+    def update_known(*self, **kwargs) -> "States":
+        """Returns a new States object, updating attributes that are already present.
+        e.g: states.update_known(**locals())"""
+        # NOTE: first argument is *self to allow the **locals() syntax inside bound methods
+        # which have their own self inside locals()
+        # otherwise will get a "got multiple values for argument 'self'" error"
+        assert len(self) == 1, "States.update_known() called with positional arguments"
+        self = self[0]
+
+        kwargs = {key: value for key, value in kwargs.items() if key in self.__dict__}
+        return self.update(**kwargs)
+
     def copy(self) -> "States":
         return jax.tree_map(lambda x: x, self)
 
@@ -328,3 +341,7 @@ class ModuleOrderError(Exception):
 
 class SubmoduleNotRegistered(Exception):
     pass
+
+
+class ModelNotInitialized(Exception):
+    pass
