@@ -1,6 +1,7 @@
 import functools
 import hashlib
 import inspect
+import io
 import os
 import re
 import shutil
@@ -14,9 +15,14 @@ import jax.numpy as jnp
 import jax.tree_util
 import numpy as np
 import toolz
+import yaml
+from rich.console import Console
 
 from elegy import types
 
+F = tp.TypeVar("F", bound=tp.Callable)
+T = tp.TypeVar("T")
+
 
 def maybe_expand_dims(a: np.ndarray, b: np.ndarray) -> tp.Tuple[np.ndarray, np.ndarray]:
     assert np.prod(a.shape) == np.prod(b.shape)
@@ -47,11 +53,28 @@ def get_signature_f_recursive(f: tp.Callable) -> tp.Callable:
         return f
 
 
+@tp.overload
 def inject_dependencies(
-    f: tp.Callable,
+    f: F,
+    rename: tp.Optional[tp.Dict[str, str]] = None,
+) -> F:
+    ...
+
+
+@tp.overload
+def inject_dependencies(
+    f: tp.Callable[..., T],
+    signature_f: tp.Callable,
+    rename: tp.Optional[tp.Dict[str, str]] = None,
+) -> tp.Callable[..., T]:
+    ...
+
+
+def inject_dependencies(
+    f: F,
     signature_f: tp.Optional[tp.Callable] = None,
     rename: tp.Optional[tp.Dict[str, str]] = None,
-):
+) -> F:
     if signature_f is None:
         signature_f = f
 
@@ -394,3 +417,57 @@ def plot_history(history):
         plt.title(title)
 
     plt.show()
+
+
+def get_grouped_entry(
+    entry: types.SummaryTableEntry,
+    depth_groups: tp.Dict[str, tp.List[types.SummaryTableEntry]],
+) -> types.SummaryTableEntry:
+    group = depth_groups[entry.path]
+
+    return types.SummaryTableEntry(
+        path=entry.path,
+        module_type_name=entry.module_type_name,
+        output_value=entry.output_value,
+        trainable_params_count=sum(entry_.trainable_params_count for entry_ in group),
+        trainable_params_size=sum(entry_.trainable_params_size for entry_ in group),
+        non_trainable_params_count=sum(
+            entry_.non_trainable_params_count for entry_ in group
+        ),
+        non_trainable_params_size=sum(
+            entry_.non_trainable_params_size for entry_ in group
+        ),
+    )
+
+
+def format_output(value) -> str:
+    file = io.StringIO()
+    outputs = jax.tree_map(
+        lambda x: f"{x.shape}" + f"{{pad}}  [dim]{x.dtype}[/]", value
+    )
+    yaml.safe_dump(
+        outputs, file, default_flow_style=False, indent=2, explicit_end=False
+    )
+    return file.getvalue().replace("\n...", "").replace("'", "")
+
+
+def format_size(size):
+    count, units = (
+        (f"{size / 1e9 :,.1f}", "GB")
+        if size > 1e9
+        else (f"{size / 1e6 :,.1f}", "MB")
+        if size > 1e6
+        else (f"{size / 1e3 :,.1f}", "KB")
+        if size > 1e3
+        else (f"{size:,}", "B")
+    )
+
+    return f"[dim]{count} {units}[/dim]"
+
+
+def get_table_repr(table):
+    f = io.StringIO()
+    console = Console(file=f, force_terminal=True)
+    console.print(table)
+
+    return f.getvalue()
